from .base_classes import *
from .support_functions import *


class List(Resource):
    """A set of information summarized from a list of other resources.If
    the element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, code=None, subject=None, source=None, date=None, ordered=None, mode=None, entry=None, emptyReason=None):
        self.original_tagname_ = None
        super(List, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.code = code
        self.subject = subject
        self.source = source
        self.date = date
        self.ordered = ordered
        self.mode = mode
        if entry is None:
            self.entry = []
        else:
            self.entry = entry
        self.emptyReason = emptyReason
    def factory(*args_, **kwargs_):
        if List.subclass:
            return List.subclass(*args_, **kwargs_)
        else:
            return List(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_source(self): return self.source
    def set_source(self, source): self.source = source
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_ordered(self): return self.ordered
    def set_ordered(self, ordered): self.ordered = ordered
    def get_mode(self): return self.mode
    def set_mode(self, mode): self.mode = mode
    def get_entry(self): return self.entry
    def set_entry(self, entry): self.entry = entry
    def add_entry(self, value): self.entry.append(value)
    def insert_entry(self, index, value): self.entry[index] = value
    def get_emptyReason(self): return self.emptyReason
    def set_emptyReason(self, emptyReason): self.emptyReason = emptyReason
    def hasContent_(self):
        if (
            self.identifier or
            self.code is not None or
            self.subject is not None or
            self.source is not None or
            self.date is not None or
            self.ordered is not None or
            self.mode is not None or
            self.entry or
            self.emptyReason is not None or
            super(List, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='List', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='List')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='List', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='List'):
        super(List, self).exportAttributes(outfile, level, already_processed, namespace_, name_='List')
    def exportChildren(self, outfile, level, namespace_='', name_='List', fromsubclass_=False, pretty_print=True):
        super(List, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.source is not None:
            self.source.export(outfile, level, namespace_, name_='source', pretty_print=pretty_print)
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.ordered is not None:
            self.ordered.export(outfile, level, namespace_, name_='ordered', pretty_print=pretty_print)
        if self.mode is not None:
            self.mode.export(outfile, level, namespace_, name_='mode', pretty_print=pretty_print)
        for entry_ in self.entry:
            entry_.export(outfile, level, namespace_, name_='entry', pretty_print=pretty_print)
        if self.emptyReason is not None:
            self.emptyReason.export(outfile, level, namespace_, name_='emptyReason', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='List', mapping_=None):
        element = super(List, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.source is not None:
            source_ = self.source
            source_.to_etree(element, name_='source', mapping_=mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.ordered is not None:
            ordered_ = self.ordered
            ordered_.to_etree(element, name_='ordered', mapping_=mapping_)
        if self.mode is not None:
            mode_ = self.mode
            mode_.to_etree(element, name_='mode', mapping_=mapping_)
        for entry_ in self.entry:
            entry_.to_etree(element, name_='entry', mapping_=mapping_)
        if self.emptyReason is not None:
            emptyReason_ = self.emptyReason
            emptyReason_.to_etree(element, name_='emptyReason', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(List, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'source':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.source = obj_
            obj_.original_tagname_ = 'source'
        elif nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'ordered':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.ordered = obj_
            obj_.original_tagname_ = 'ordered'
        elif nodeName_ == 'mode':
            obj_ = ListMode.factory()
            obj_.build(child_)
            self.mode = obj_
            obj_.original_tagname_ = 'mode'
        elif nodeName_ == 'entry':
            obj_ = List_Entry.factory()
            obj_.build(child_)
            self.entry.append(obj_)
            obj_.original_tagname_ = 'entry'
        elif nodeName_ == 'emptyReason':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.emptyReason = obj_
            obj_.original_tagname_ = 'emptyReason'
        super(List, self).buildChildren(child_, node, nodeName_, True)
# end class List


class List_Entry(BackboneElement):
    """A set of information summarized from a list of other resources."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, flag=None, deleted=None, date=None, item=None):
        self.original_tagname_ = None
        super(List_Entry, self).__init__(id, extension, modifierExtension, )
        if flag is None:
            self.flag = []
        else:
            self.flag = flag
        self.deleted = deleted
        self.date = date
        self.item = item
    def factory(*args_, **kwargs_):
        if List_Entry.subclass:
            return List_Entry.subclass(*args_, **kwargs_)
        else:
            return List_Entry(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_flag(self): return self.flag
    def set_flag(self, flag): self.flag = flag
    def add_flag(self, value): self.flag.append(value)
    def insert_flag(self, index, value): self.flag[index] = value
    def get_deleted(self): return self.deleted
    def set_deleted(self, deleted): self.deleted = deleted
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_item(self): return self.item
    def set_item(self, item): self.item = item
    def hasContent_(self):
        if (
            self.flag or
            self.deleted is not None or
            self.date is not None or
            self.item is not None or
            super(List_Entry, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='List.Entry', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='List.Entry')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='List.Entry', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='List.Entry'):
        super(List_Entry, self).exportAttributes(outfile, level, already_processed, namespace_, name_='List.Entry')
    def exportChildren(self, outfile, level, namespace_='', name_='List.Entry', fromsubclass_=False, pretty_print=True):
        super(List_Entry, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for flag_ in self.flag:
            flag_.export(outfile, level, namespace_, name_='flag', pretty_print=pretty_print)
        if self.deleted is not None:
            self.deleted.export(outfile, level, namespace_, name_='deleted', pretty_print=pretty_print)
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.item is not None:
            self.item.export(outfile, level, namespace_, name_='item', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='List.Entry', mapping_=None):
        element = super(List_Entry, self).to_etree(parent_element, name_, mapping_)
        for flag_ in self.flag:
            flag_.to_etree(element, name_='flag', mapping_=mapping_)
        if self.deleted is not None:
            deleted_ = self.deleted
            deleted_.to_etree(element, name_='deleted', mapping_=mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.item is not None:
            item_ = self.item
            item_.to_etree(element, name_='item', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(List_Entry, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'flag':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.flag.append(obj_)
            obj_.original_tagname_ = 'flag'
        elif nodeName_ == 'deleted':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.deleted = obj_
            obj_.original_tagname_ = 'deleted'
        elif nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'item':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.item = obj_
            obj_.original_tagname_ = 'item'
        super(List_Entry, self).buildChildren(child_, node, nodeName_, True)
# end class List_Entry


class ListMode(Element):
    """The processing mode that applies to this listIf the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ListMode, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ListMode.subclass:
            return ListMode.subclass(*args_, **kwargs_)
        else:
            return ListMode(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ListMode, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ListMode', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ListMode')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ListMode', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ListMode'):
        super(ListMode, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ListMode')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ListMode', fromsubclass_=False, pretty_print=True):
        super(ListMode, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ListMode', mapping_=None):
        element = super(ListMode, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ListMode, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ListMode, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ListMode



from .base_classes import *
from .support_functions import *


class MedicationPrescription(Resource):
    """An order for both supply of the medication and the instructions for
    administration of the medicine to a patient.If the element is
    present, it must have either a @value, an @id, or extensionsCan
    be the reason or the indication for writing the prescription."""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, dateWritten=None, status=None, patient=None, prescriber=None, encounter=None, reasonCodeableConcept=None, reasonResource=None, medication=None, dosageInstruction=None, dispense=None, substitution=None):
        self.original_tagname_ = None
        super(MedicationPrescription, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.dateWritten = dateWritten
        self.status = status
        self.patient = patient
        self.prescriber = prescriber
        self.encounter = encounter
        self.reasonCodeableConcept = reasonCodeableConcept
        self.reasonResource = reasonResource
        self.medication = medication
        if dosageInstruction is None:
            self.dosageInstruction = []
        else:
            self.dosageInstruction = dosageInstruction
        self.dispense = dispense
        self.substitution = substitution
    def factory(*args_, **kwargs_):
        if MedicationPrescription.subclass:
            return MedicationPrescription.subclass(*args_, **kwargs_)
        else:
            return MedicationPrescription(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_dateWritten(self): return self.dateWritten
    def set_dateWritten(self, dateWritten): self.dateWritten = dateWritten
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_patient(self): return self.patient
    def set_patient(self, patient): self.patient = patient
    def get_prescriber(self): return self.prescriber
    def set_prescriber(self, prescriber): self.prescriber = prescriber
    def get_encounter(self): return self.encounter
    def set_encounter(self, encounter): self.encounter = encounter
    def get_reasonCodeableConcept(self): return self.reasonCodeableConcept
    def set_reasonCodeableConcept(self, reasonCodeableConcept): self.reasonCodeableConcept = reasonCodeableConcept
    def get_reasonResource(self): return self.reasonResource
    def set_reasonResource(self, reasonResource): self.reasonResource = reasonResource
    def get_medication(self): return self.medication
    def set_medication(self, medication): self.medication = medication
    def get_dosageInstruction(self): return self.dosageInstruction
    def set_dosageInstruction(self, dosageInstruction): self.dosageInstruction = dosageInstruction
    def add_dosageInstruction(self, value): self.dosageInstruction.append(value)
    def insert_dosageInstruction(self, index, value): self.dosageInstruction[index] = value
    def get_dispense(self): return self.dispense
    def set_dispense(self, dispense): self.dispense = dispense
    def get_substitution(self): return self.substitution
    def set_substitution(self, substitution): self.substitution = substitution
    def hasContent_(self):
        if (
            self.identifier or
            self.dateWritten is not None or
            self.status is not None or
            self.patient is not None or
            self.prescriber is not None or
            self.encounter is not None or
            self.reasonCodeableConcept is not None or
            self.reasonResource is not None or
            self.medication is not None or
            self.dosageInstruction or
            self.dispense is not None or
            self.substitution is not None or
            super(MedicationPrescription, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationPrescription', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationPrescription')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationPrescription', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationPrescription'):
        super(MedicationPrescription, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationPrescription')
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationPrescription', fromsubclass_=False, pretty_print=True):
        super(MedicationPrescription, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.dateWritten is not None:
            self.dateWritten.export(outfile, level, namespace_, name_='dateWritten', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.patient is not None:
            self.patient.export(outfile, level, namespace_, name_='patient', pretty_print=pretty_print)
        if self.prescriber is not None:
            self.prescriber.export(outfile, level, namespace_, name_='prescriber', pretty_print=pretty_print)
        if self.encounter is not None:
            self.encounter.export(outfile, level, namespace_, name_='encounter', pretty_print=pretty_print)
        if self.reasonCodeableConcept is not None:
            self.reasonCodeableConcept.export(outfile, level, namespace_, name_='reasonCodeableConcept', pretty_print=pretty_print)
        if self.reasonResource is not None:
            self.reasonResource.export(outfile, level, namespace_, name_='reasonResource', pretty_print=pretty_print)
        if self.medication is not None:
            self.medication.export(outfile, level, namespace_, name_='medication', pretty_print=pretty_print)
        for dosageInstruction_ in self.dosageInstruction:
            dosageInstruction_.export(outfile, level, namespace_, name_='dosageInstruction', pretty_print=pretty_print)
        if self.dispense is not None:
            self.dispense.export(outfile, level, namespace_, name_='dispense', pretty_print=pretty_print)
        if self.substitution is not None:
            self.substitution.export(outfile, level, namespace_, name_='substitution', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationPrescription', mapping_=None):
        element = super(MedicationPrescription, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.dateWritten is not None:
            dateWritten_ = self.dateWritten
            dateWritten_.to_etree(element, name_='dateWritten', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.patient is not None:
            patient_ = self.patient
            patient_.to_etree(element, name_='patient', mapping_=mapping_)
        if self.prescriber is not None:
            prescriber_ = self.prescriber
            prescriber_.to_etree(element, name_='prescriber', mapping_=mapping_)
        if self.encounter is not None:
            encounter_ = self.encounter
            encounter_.to_etree(element, name_='encounter', mapping_=mapping_)
        if self.reasonCodeableConcept is not None:
            reasonCodeableConcept_ = self.reasonCodeableConcept
            reasonCodeableConcept_.to_etree(element, name_='reasonCodeableConcept', mapping_=mapping_)
        if self.reasonResource is not None:
            reasonResource_ = self.reasonResource
            reasonResource_.to_etree(element, name_='reasonResource', mapping_=mapping_)
        if self.medication is not None:
            medication_ = self.medication
            medication_.to_etree(element, name_='medication', mapping_=mapping_)
        for dosageInstruction_ in self.dosageInstruction:
            dosageInstruction_.to_etree(element, name_='dosageInstruction', mapping_=mapping_)
        if self.dispense is not None:
            dispense_ = self.dispense
            dispense_.to_etree(element, name_='dispense', mapping_=mapping_)
        if self.substitution is not None:
            substitution_ = self.substitution
            substitution_.to_etree(element, name_='substitution', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MedicationPrescription, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'dateWritten':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.dateWritten = obj_
            obj_.original_tagname_ = 'dateWritten'
        elif nodeName_ == 'status':
            obj_ = MedicationPrescriptionStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'patient':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.patient = obj_
            obj_.original_tagname_ = 'patient'
        elif nodeName_ == 'prescriber':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.prescriber = obj_
            obj_.original_tagname_ = 'prescriber'
        elif nodeName_ == 'encounter':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.encounter = obj_
            obj_.original_tagname_ = 'encounter'
        elif nodeName_ == 'reasonCodeableConcept':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.reasonCodeableConcept = obj_
            obj_.original_tagname_ = 'reasonCodeableConcept'
        elif nodeName_ == 'reasonResource':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.reasonResource = obj_
            obj_.original_tagname_ = 'reasonResource'
        elif nodeName_ == 'medication':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.medication = obj_
            obj_.original_tagname_ = 'medication'
        elif nodeName_ == 'dosageInstruction':
            obj_ = MedicationPrescription_DosageInstruction.factory()
            obj_.build(child_)
            self.dosageInstruction.append(obj_)
            obj_.original_tagname_ = 'dosageInstruction'
        elif nodeName_ == 'dispense':
            obj_ = MedicationPrescription_Dispense.factory()
            obj_.build(child_)
            self.dispense = obj_
            obj_.original_tagname_ = 'dispense'
        elif nodeName_ == 'substitution':
            obj_ = MedicationPrescription_Substitution.factory()
            obj_.build(child_)
            self.substitution = obj_
            obj_.original_tagname_ = 'substitution'
        super(MedicationPrescription, self).buildChildren(child_, node, nodeName_, True)
# end class MedicationPrescription


class MedicationPrescription_DosageInstruction(BackboneElement):
    """An order for both supply of the medication and the instructions for
    administration of the medicine to a patient.The timing schedule
    for giving the medication to the patient. The Schedule data type
    allows many different expressions, for example. "Every 8 hours";
    "Three times a day"; "1/2 an hour before breakfast for 10 days
    from 23-Dec 2011:"; "15 Oct 2013, 17 Oct 2013 and 1 Nov 2013".If
    set to true or if specified as a CodeableConcept, indicates that
    the medication is only taken when needed within the specified
    schedule rather than at every scheduled dose. If a
    CodeableConcept is present, it indicates the pre-condition for
    taking the Medication."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, text=None, additionalInstructions=None, timingDateTime=None, timingPeriod=None, timingSchedule=None, asNeededBoolean=None, asNeededCodeableConcept=None, site=None, route=None, method=None, doseQuantity=None, rate=None, maxDosePerPeriod=None):
        self.original_tagname_ = None
        super(MedicationPrescription_DosageInstruction, self).__init__(id, extension, modifierExtension, )
        self.text = text
        self.additionalInstructions = additionalInstructions
        self.timingDateTime = timingDateTime
        self.timingPeriod = timingPeriod
        self.timingSchedule = timingSchedule
        self.asNeededBoolean = asNeededBoolean
        self.asNeededCodeableConcept = asNeededCodeableConcept
        self.site = site
        self.route = route
        self.method = method
        self.doseQuantity = doseQuantity
        self.rate = rate
        self.maxDosePerPeriod = maxDosePerPeriod
    def factory(*args_, **kwargs_):
        if MedicationPrescription_DosageInstruction.subclass:
            return MedicationPrescription_DosageInstruction.subclass(*args_, **kwargs_)
        else:
            return MedicationPrescription_DosageInstruction(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_text(self): return self.text
    def set_text(self, text): self.text = text
    def get_additionalInstructions(self): return self.additionalInstructions
    def set_additionalInstructions(self, additionalInstructions): self.additionalInstructions = additionalInstructions
    def get_timingDateTime(self): return self.timingDateTime
    def set_timingDateTime(self, timingDateTime): self.timingDateTime = timingDateTime
    def get_timingPeriod(self): return self.timingPeriod
    def set_timingPeriod(self, timingPeriod): self.timingPeriod = timingPeriod
    def get_timingSchedule(self): return self.timingSchedule
    def set_timingSchedule(self, timingSchedule): self.timingSchedule = timingSchedule
    def get_asNeededBoolean(self): return self.asNeededBoolean
    def set_asNeededBoolean(self, asNeededBoolean): self.asNeededBoolean = asNeededBoolean
    def get_asNeededCodeableConcept(self): return self.asNeededCodeableConcept
    def set_asNeededCodeableConcept(self, asNeededCodeableConcept): self.asNeededCodeableConcept = asNeededCodeableConcept
    def get_site(self): return self.site
    def set_site(self, site): self.site = site
    def get_route(self): return self.route
    def set_route(self, route): self.route = route
    def get_method(self): return self.method
    def set_method(self, method): self.method = method
    def get_doseQuantity(self): return self.doseQuantity
    def set_doseQuantity(self, doseQuantity): self.doseQuantity = doseQuantity
    def get_rate(self): return self.rate
    def set_rate(self, rate): self.rate = rate
    def get_maxDosePerPeriod(self): return self.maxDosePerPeriod
    def set_maxDosePerPeriod(self, maxDosePerPeriod): self.maxDosePerPeriod = maxDosePerPeriod
    def hasContent_(self):
        if (
            self.text is not None or
            self.additionalInstructions is not None or
            self.timingDateTime is not None or
            self.timingPeriod is not None or
            self.timingSchedule is not None or
            self.asNeededBoolean is not None or
            self.asNeededCodeableConcept is not None or
            self.site is not None or
            self.route is not None or
            self.method is not None or
            self.doseQuantity is not None or
            self.rate is not None or
            self.maxDosePerPeriod is not None or
            super(MedicationPrescription_DosageInstruction, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationPrescription.DosageInstruction', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationPrescription.DosageInstruction')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationPrescription.DosageInstruction', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationPrescription.DosageInstruction'):
        super(MedicationPrescription_DosageInstruction, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationPrescription.DosageInstruction')
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationPrescription.DosageInstruction', fromsubclass_=False, pretty_print=True):
        super(MedicationPrescription_DosageInstruction, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.text is not None:
            self.text.export(outfile, level, namespace_, name_='text', pretty_print=pretty_print)
        if self.additionalInstructions is not None:
            self.additionalInstructions.export(outfile, level, namespace_, name_='additionalInstructions', pretty_print=pretty_print)
        if self.timingDateTime is not None:
            self.timingDateTime.export(outfile, level, namespace_, name_='timingDateTime', pretty_print=pretty_print)
        if self.timingPeriod is not None:
            self.timingPeriod.export(outfile, level, namespace_, name_='timingPeriod', pretty_print=pretty_print)
        if self.timingSchedule is not None:
            self.timingSchedule.export(outfile, level, namespace_, name_='timingSchedule', pretty_print=pretty_print)
        if self.asNeededBoolean is not None:
            self.asNeededBoolean.export(outfile, level, namespace_, name_='asNeededBoolean', pretty_print=pretty_print)
        if self.asNeededCodeableConcept is not None:
            self.asNeededCodeableConcept.export(outfile, level, namespace_, name_='asNeededCodeableConcept', pretty_print=pretty_print)
        if self.site is not None:
            self.site.export(outfile, level, namespace_, name_='site', pretty_print=pretty_print)
        if self.route is not None:
            self.route.export(outfile, level, namespace_, name_='route', pretty_print=pretty_print)
        if self.method is not None:
            self.method.export(outfile, level, namespace_, name_='method', pretty_print=pretty_print)
        if self.doseQuantity is not None:
            self.doseQuantity.export(outfile, level, namespace_, name_='doseQuantity', pretty_print=pretty_print)
        if self.rate is not None:
            self.rate.export(outfile, level, namespace_, name_='rate', pretty_print=pretty_print)
        if self.maxDosePerPeriod is not None:
            self.maxDosePerPeriod.export(outfile, level, namespace_, name_='maxDosePerPeriod', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationPrescription.DosageInstruction', mapping_=None):
        element = super(MedicationPrescription_DosageInstruction, self).to_etree(parent_element, name_, mapping_)
        if self.text is not None:
            text_ = self.text
            text_.to_etree(element, name_='text', mapping_=mapping_)
        if self.additionalInstructions is not None:
            additionalInstructions_ = self.additionalInstructions
            additionalInstructions_.to_etree(element, name_='additionalInstructions', mapping_=mapping_)
        if self.timingDateTime is not None:
            timingDateTime_ = self.timingDateTime
            timingDateTime_.to_etree(element, name_='timingDateTime', mapping_=mapping_)
        if self.timingPeriod is not None:
            timingPeriod_ = self.timingPeriod
            timingPeriod_.to_etree(element, name_='timingPeriod', mapping_=mapping_)
        if self.timingSchedule is not None:
            timingSchedule_ = self.timingSchedule
            timingSchedule_.to_etree(element, name_='timingSchedule', mapping_=mapping_)
        if self.asNeededBoolean is not None:
            asNeededBoolean_ = self.asNeededBoolean
            asNeededBoolean_.to_etree(element, name_='asNeededBoolean', mapping_=mapping_)
        if self.asNeededCodeableConcept is not None:
            asNeededCodeableConcept_ = self.asNeededCodeableConcept
            asNeededCodeableConcept_.to_etree(element, name_='asNeededCodeableConcept', mapping_=mapping_)
        if self.site is not None:
            site_ = self.site
            site_.to_etree(element, name_='site', mapping_=mapping_)
        if self.route is not None:
            route_ = self.route
            route_.to_etree(element, name_='route', mapping_=mapping_)
        if self.method is not None:
            method_ = self.method
            method_.to_etree(element, name_='method', mapping_=mapping_)
        if self.doseQuantity is not None:
            doseQuantity_ = self.doseQuantity
            doseQuantity_.to_etree(element, name_='doseQuantity', mapping_=mapping_)
        if self.rate is not None:
            rate_ = self.rate
            rate_.to_etree(element, name_='rate', mapping_=mapping_)
        if self.maxDosePerPeriod is not None:
            maxDosePerPeriod_ = self.maxDosePerPeriod
            maxDosePerPeriod_.to_etree(element, name_='maxDosePerPeriod', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MedicationPrescription_DosageInstruction, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'text':
            obj_ = string.factory()
            obj_.build(child_)
            self.text = obj_
            obj_.original_tagname_ = 'text'
        elif nodeName_ == 'additionalInstructions':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.additionalInstructions = obj_
            obj_.original_tagname_ = 'additionalInstructions'
        elif nodeName_ == 'timingDateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.timingDateTime = obj_
            obj_.original_tagname_ = 'timingDateTime'
        elif nodeName_ == 'timingPeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.timingPeriod = obj_
            obj_.original_tagname_ = 'timingPeriod'
        elif nodeName_ == 'timingSchedule':
            obj_ = Schedule.factory()
            obj_.build(child_)
            self.timingSchedule = obj_
            obj_.original_tagname_ = 'timingSchedule'
        elif nodeName_ == 'asNeededBoolean':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.asNeededBoolean = obj_
            obj_.original_tagname_ = 'asNeededBoolean'
        elif nodeName_ == 'asNeededCodeableConcept':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.asNeededCodeableConcept = obj_
            obj_.original_tagname_ = 'asNeededCodeableConcept'
        elif nodeName_ == 'site':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.site = obj_
            obj_.original_tagname_ = 'site'
        elif nodeName_ == 'route':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.route = obj_
            obj_.original_tagname_ = 'route'
        elif nodeName_ == 'method':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.method = obj_
            obj_.original_tagname_ = 'method'
        elif nodeName_ == 'doseQuantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.doseQuantity = obj_
            obj_.original_tagname_ = 'doseQuantity'
        elif nodeName_ == 'rate':
            obj_ = Ratio.factory()
            obj_.build(child_)
            self.rate = obj_
            obj_.original_tagname_ = 'rate'
        elif nodeName_ == 'maxDosePerPeriod':
            obj_ = Ratio.factory()
            obj_.build(child_)
            self.maxDosePerPeriod = obj_
            obj_.original_tagname_ = 'maxDosePerPeriod'
        super(MedicationPrescription_DosageInstruction, self).buildChildren(child_, node, nodeName_, True)
# end class MedicationPrescription_DosageInstruction


class MedicationPrescription_Dispense(BackboneElement):
    """An order for both supply of the medication and the instructions for
    administration of the medicine to a patient."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, medication=None, validityPeriod=None, numberOfRepeatsAllowed=None, quantity=None, expectedSupplyDuration=None):
        self.original_tagname_ = None
        super(MedicationPrescription_Dispense, self).__init__(id, extension, modifierExtension, )
        self.medication = medication
        self.validityPeriod = validityPeriod
        self.numberOfRepeatsAllowed = numberOfRepeatsAllowed
        self.quantity = quantity
        self.expectedSupplyDuration = expectedSupplyDuration
    def factory(*args_, **kwargs_):
        if MedicationPrescription_Dispense.subclass:
            return MedicationPrescription_Dispense.subclass(*args_, **kwargs_)
        else:
            return MedicationPrescription_Dispense(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_medication(self): return self.medication
    def set_medication(self, medication): self.medication = medication
    def get_validityPeriod(self): return self.validityPeriod
    def set_validityPeriod(self, validityPeriod): self.validityPeriod = validityPeriod
    def get_numberOfRepeatsAllowed(self): return self.numberOfRepeatsAllowed
    def set_numberOfRepeatsAllowed(self, numberOfRepeatsAllowed): self.numberOfRepeatsAllowed = numberOfRepeatsAllowed
    def get_quantity(self): return self.quantity
    def set_quantity(self, quantity): self.quantity = quantity
    def get_expectedSupplyDuration(self): return self.expectedSupplyDuration
    def set_expectedSupplyDuration(self, expectedSupplyDuration): self.expectedSupplyDuration = expectedSupplyDuration
    def hasContent_(self):
        if (
            self.medication is not None or
            self.validityPeriod is not None or
            self.numberOfRepeatsAllowed is not None or
            self.quantity is not None or
            self.expectedSupplyDuration is not None or
            super(MedicationPrescription_Dispense, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationPrescription.Dispense', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationPrescription.Dispense')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationPrescription.Dispense', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationPrescription.Dispense'):
        super(MedicationPrescription_Dispense, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationPrescription.Dispense')
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationPrescription.Dispense', fromsubclass_=False, pretty_print=True):
        super(MedicationPrescription_Dispense, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.medication is not None:
            self.medication.export(outfile, level, namespace_, name_='medication', pretty_print=pretty_print)
        if self.validityPeriod is not None:
            self.validityPeriod.export(outfile, level, namespace_, name_='validityPeriod', pretty_print=pretty_print)
        if self.numberOfRepeatsAllowed is not None:
            self.numberOfRepeatsAllowed.export(outfile, level, namespace_, name_='numberOfRepeatsAllowed', pretty_print=pretty_print)
        if self.quantity is not None:
            self.quantity.export(outfile, level, namespace_, name_='quantity', pretty_print=pretty_print)
        if self.expectedSupplyDuration is not None:
            self.expectedSupplyDuration.export(outfile, level, namespace_, name_='expectedSupplyDuration', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationPrescription.Dispense', mapping_=None):
        element = super(MedicationPrescription_Dispense, self).to_etree(parent_element, name_, mapping_)
        if self.medication is not None:
            medication_ = self.medication
            medication_.to_etree(element, name_='medication', mapping_=mapping_)
        if self.validityPeriod is not None:
            validityPeriod_ = self.validityPeriod
            validityPeriod_.to_etree(element, name_='validityPeriod', mapping_=mapping_)
        if self.numberOfRepeatsAllowed is not None:
            numberOfRepeatsAllowed_ = self.numberOfRepeatsAllowed
            numberOfRepeatsAllowed_.to_etree(element, name_='numberOfRepeatsAllowed', mapping_=mapping_)
        if self.quantity is not None:
            quantity_ = self.quantity
            quantity_.to_etree(element, name_='quantity', mapping_=mapping_)
        if self.expectedSupplyDuration is not None:
            expectedSupplyDuration_ = self.expectedSupplyDuration
            expectedSupplyDuration_.to_etree(element, name_='expectedSupplyDuration', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MedicationPrescription_Dispense, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'medication':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.medication = obj_
            obj_.original_tagname_ = 'medication'
        elif nodeName_ == 'validityPeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.validityPeriod = obj_
            obj_.original_tagname_ = 'validityPeriod'
        elif nodeName_ == 'numberOfRepeatsAllowed':
            obj_ = integer.factory()
            obj_.build(child_)
            self.numberOfRepeatsAllowed = obj_
            obj_.original_tagname_ = 'numberOfRepeatsAllowed'
        elif nodeName_ == 'quantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.quantity = obj_
            obj_.original_tagname_ = 'quantity'
        elif nodeName_ == 'expectedSupplyDuration':
            obj_ = Duration.factory()
            obj_.build(child_)
            self.expectedSupplyDuration = obj_
            obj_.original_tagname_ = 'expectedSupplyDuration'
        super(MedicationPrescription_Dispense, self).buildChildren(child_, node, nodeName_, True)
# end class MedicationPrescription_Dispense


class MedicationPrescription_Substitution(BackboneElement):
    """An order for both supply of the medication and the instructions for
    administration of the medicine to a patient."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, type_=None, reason=None):
        self.original_tagname_ = None
        super(MedicationPrescription_Substitution, self).__init__(id, extension, modifierExtension, )
        self.type_ = type_
        self.reason = reason
    def factory(*args_, **kwargs_):
        if MedicationPrescription_Substitution.subclass:
            return MedicationPrescription_Substitution.subclass(*args_, **kwargs_)
        else:
            return MedicationPrescription_Substitution(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_reason(self): return self.reason
    def set_reason(self, reason): self.reason = reason
    def hasContent_(self):
        if (
            self.type_ is not None or
            self.reason is not None or
            super(MedicationPrescription_Substitution, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationPrescription.Substitution', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationPrescription.Substitution')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationPrescription.Substitution', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationPrescription.Substitution'):
        super(MedicationPrescription_Substitution, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationPrescription.Substitution')
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationPrescription.Substitution', fromsubclass_=False, pretty_print=True):
        super(MedicationPrescription_Substitution, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.reason is not None:
            self.reason.export(outfile, level, namespace_, name_='reason', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationPrescription.Substitution', mapping_=None):
        element = super(MedicationPrescription_Substitution, self).to_etree(parent_element, name_, mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.reason is not None:
            reason_ = self.reason
            reason_.to_etree(element, name_='reason', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MedicationPrescription_Substitution, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'reason':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.reason = obj_
            obj_.original_tagname_ = 'reason'
        super(MedicationPrescription_Substitution, self).buildChildren(child_, node, nodeName_, True)
# end class MedicationPrescription_Substitution


class MedicationPrescriptionStatus(Element):
    """A code specifying the state of the prescribing event. Describes the
    lifecycle of the prescription.If the element is present, it must
    have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(MedicationPrescriptionStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if MedicationPrescriptionStatus.subclass:
            return MedicationPrescriptionStatus.subclass(*args_, **kwargs_)
        else:
            return MedicationPrescriptionStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(MedicationPrescriptionStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationPrescriptionStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationPrescriptionStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationPrescriptionStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationPrescriptionStatus'):
        super(MedicationPrescriptionStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationPrescriptionStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationPrescriptionStatus', fromsubclass_=False, pretty_print=True):
        super(MedicationPrescriptionStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationPrescriptionStatus', mapping_=None):
        element = super(MedicationPrescriptionStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(MedicationPrescriptionStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(MedicationPrescriptionStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class MedicationPrescriptionStatus



from configparser import SafeConfigParser

class Configuration:
    def __init__(self, fileName):
        cp = SafeConfigParser()
        cp.read(fileName)
        self.__parser = cp
        self.fileName = fileName

    def __getattr__(self, name):
        if name in self.__parser.sections():
            return Section(name, self.__parser)
        else:
            return None

    def __str__(self):
        p = self.__parser
        result = []
        result.append('<Configuration from %s>' % self.fileName)
        for s in p.sections():
            result.append('[%s]' % s)
            for o in p.options(s):
                result.append('%s=%s' % (o, p.get(s, o)))
        return '\n'.join(result)

class Section:
    def __init__(self, name, parser):
        self.name = name
        self.__parser = parser

    def __getattr__(self, name):
        return self.__parser.get(self.name, name)

    def __str__(self):
        return str(self.__repr__())

    def __repr__(self):
        return self.__parser.items(self.name)

    def getboolean(self, name):
        return self.__parser.getboolean(self.name, name)

    def __contains__(self, name):
        return self.__parser.has_option(self.name, name)

    def get(self, name, default):
        if name in self:
            return self.__getattr__(name)
        else:
            return default

    def set(self, name, value):
        self.__parser.set(self.name, name, str(value))

# Test
if __name__ == '__main__':
    c = Configuration('Importador.ini')
    print((c.Origem.host, c.Origem.port))



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import datetime
from trytond.model import ModelView, ModelSingleton, ModelSQL, fields, Unique
from trytond.transaction import Transaction
from trytond.tools import grouped_slice, reduce_ids
from sql import Literal, Table
from trytond.pool import Pool
from trytond.pyson import Eval, Not, Bool, And, Equal, Or
from trytond import backend
import pytz


__all__ = ['InpatientSequences', 'DietTherapeutic','InpatientRegistration', 
    'BedTransfer', 'Appointment', 'PatientEvaluation', 'PatientData',
    'InpatientMedication', 'InpatientMedicationAdminTimes',
    'InpatientMedicationLog', 'InpatientDiet', 'InpatientMeal',
    'InpatientMealOrder','InpatientMealOrderItem', 'ECG']


class InpatientSequences(ModelSingleton, ModelSQL, ModelView):
    "Inpatient Registration Sequences for GNU Health"
    __name__ = "gnuhealth.sequences"

    inpatient_registration_sequence = fields.Property(fields.Many2One(
        'ir.sequence', 'Inpatient Sequence', required=True,
        domain=[('code', '=', 'gnuhealth.inpatient.registration')]))

    inpatient_meal_order_sequence = fields.Property(fields.Many2One(
        'ir.sequence', 'Inpatient Meal Sequence', required=True,
        domain=[('code', '=', 'gnuhealth.inpatient.meal.order')]))


# Therapeutic Diet types

class DietTherapeutic (ModelSQL, ModelView):
    'Diet Therapy'
    __name__="gnuhealth.diet.therapeutic"

    name = fields.Char('Diet type', required=True, translate=True)
    code = fields.Char('Code', required=True)
    description = fields.Text('Indications', required=True, translate=True)

    @classmethod
    def __setup__(cls):
        super(DietTherapeutic, cls).__setup__()

        t = cls.__table__()
        cls._sql_constraints = [
            ('code_unique', Unique(t,t.code),
                'The Diet code already exists'),
            ]


class InpatientRegistration(ModelSQL, ModelView):
    'Patient admission History'
    __name__ = 'gnuhealth.inpatient.registration'

    STATES = {'readonly': Or(Eval('state') == 'done',
        Eval('state') == 'finished')}
    
    name = fields.Char('Registration Code', readonly=True, select=True)
    patient = fields.Many2One('gnuhealth.patient', 'Patient',
     required=True, select=True, states = STATES)
    admission_type = fields.Selection([
        (None, ''),
        ('routine', 'Routine'),
        ('maternity', 'Maternity'),
        ('elective', 'Elective'),
        ('urgent', 'Urgent'),
        ('emergency', 'Emergency'),
        ], 'Admission type', required=True, select=True, states = STATES)
    hospitalization_date = fields.DateTime('Hospitalization date',
        required=True, select=True, states = STATES)
    discharge_date = fields.DateTime('Expected Discharge Date', required=True,
        states = STATES)
    attending_physician = fields.Many2One('gnuhealth.healthprofessional',
        'Attending Physician',  states = STATES)
    operating_physician = fields.Many2One('gnuhealth.healthprofessional',
        'Operating Physician',  states = STATES)
    admission_reason = fields.Many2One('gnuhealth.pathology',
        'Reason for Admission', help="Reason for Admission",  states = STATES,
        select=True)
    bed = fields.Many2One('gnuhealth.hospital.bed', 'Hospital Bed',
        states={
            'required': Not(Bool(Eval('name'))),
            'readonly': Or(Eval('state') == 'done',
                            Eval('state') == 'finished', 
                            Bool(Eval('name')),
                        )
            },
        depends=['name'])
    nursing_plan = fields.Text('Nursing Plan', states = STATES)
    medications = fields.One2Many('gnuhealth.inpatient.medication', 'name',
        'Medications', states = STATES)
    therapeutic_diets = fields.One2Many('gnuhealth.inpatient.diet', 'name',
        'Meals / Diet Program', states = STATES)
        
    nutrition_notes = fields.Text('Nutrition notes / directions', states = STATES)
    discharge_plan = fields.Text('Discharge Plan', states = STATES)
    info = fields.Text('Notes',states = STATES)
    state = fields.Selection((
        (None, ''),
        ('free', 'free'),
        ('cancelled', 'cancelled'),
        ('confirmed', 'confirmed'),
        ('hospitalized', 'hospitalized'),
        ('done','Discharged - needs cleaning'),
        ('finished','Finished'),
        ), 'Status', select=True, readonly=True)
        
    bed_transfers = fields.One2Many('gnuhealth.bed.transfer', 'name',
        'Transfer History', readonly=True)

    discharged_by = fields.Many2One(
        'gnuhealth.healthprofessional', 'Discharged by', readonly=True,
        help="Health Professional that discharged the patient")

    discharge_reason = fields.Selection([
        (None, ''),
        ('home','Home / Selfcare'),
        ('transfer','Transferred to another institution'),
        ('death','Death'),
        ('against_advice','Left against medical advice')],
        'Discharge Reason', 
        states={'readonly': Not(Equal(Eval('state'), 'hospitalized')),},
        help="Reason for patient discharge")

    discharge_dx = fields.Many2One('gnuhealth.pathology',
        'Discharge Dx', help="Code for Discharge Diagnosis",
        states={'readonly': Not(Equal(Eval('state'), 'hospitalized')),})
  
    institution = fields.Many2One('gnuhealth.institution', 'Institution',
        readonly=True)

    puid = fields.Function(
        fields.Char('PUID', help="Person Unique Identifier"),
        'get_patient_puid', searcher="search_patient_puid")



    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        institution = HealthInst.get_institution()
        return institution

    def get_patient_puid(self, name):
        return self.patient.name.ref

    @classmethod
    def search_patient_puid(cls, name, clause):
        res = []
        value = clause[2]
        res.append(('patient.name.ref', clause[1], value))
        return res

    @classmethod
    def __setup__(cls):
        super(InpatientRegistration, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('name_unique', Unique(t,t.name),
                'The Registration code already exists'),
            ]

        cls._error_messages.update({
                'bed_is_not_available': 'Bed is not available',
                'discharge_reason_needed': 'Admission and Discharge reasons \n'
                'as well as Discharge Dx are needed',
                'destination_bed_unavailable': 'Destination bed unavailable',
                'need_timezone':'You need to set up the company timezone'})
        cls._buttons.update({
                'confirmed': {
                    'invisible': And(Not(Equal(Eval('state'), 'free')),
                        Not(Equal(Eval('state'), 'cancelled'))),
                    },
                'cancel': {
                    'invisible': Not(Equal(Eval('state'), 'confirmed')),
                    },
                'admission': {
                    'invisible': Not(Equal(Eval('state'), 'confirmed')),
                    },
                'discharge': {
                    'invisible': Not(Equal(Eval('state'), 'hospitalized')),
                    },
                'bedclean': {
                    'invisible': Not(Equal(Eval('state'), 'done')),
                    },
                })

    ## Method to check for availability and make the hospital bed reservation
    # Checks that there are not overlapping dates and status of the bed / room
    # is not confirmed, hospitalized or done but requiring cleaning ('done')
    @classmethod
    @ModelView.button
    def confirmed(cls, registrations):
        registration_id = registrations[0]
        Bed = Pool().get('gnuhealth.hospital.bed')
        cursor = Transaction().connection.cursor()
        bed_id = registration_id.bed.id
        cursor.execute("SELECT COUNT(*) \
            FROM gnuhealth_inpatient_registration \
            WHERE (hospitalization_date::timestamp,discharge_date::timestamp) \
                OVERLAPS (timestamp %s, timestamp %s) \
              AND (state = %s or state = %s or state = %s) \
              AND bed = CAST(%s AS INTEGER) ",
            (registration_id.hospitalization_date,
            registration_id.discharge_date,
            'confirmed', 'hospitalized', 'done', str(bed_id)))
        res = cursor.fetchone()
        if (registration_id.discharge_date.date() <
            registration_id.hospitalization_date.date()):
            cls.raise_user_error("The Discharge date must later than the " \
                "Admission")
        if res[0] > 0:
            cls.raise_user_error('bed_is_not_available')
        else:
            cls.write(registrations, {'state': 'confirmed'})
            Bed.write([registration_id.bed], {'state': 'reserved'})

    @classmethod
    @ModelView.button
    def discharge(cls, registrations):
        registration_id = registrations[0]
        Bed = Pool().get('gnuhealth.hospital.bed')

        signing_hp = Pool().get('gnuhealth.healthprofessional').get_health_professional()
        if not signing_hp:
            cls.raise_user_error(
                "No health professional associated to this user !")

        cls.write(registrations, {'state': 'done',
            'discharged_by': signing_hp})

        Bed.write([registration_id.bed], {'state': 'to_clean'})

    @classmethod
    @ModelView.button
    def bedclean(cls, registrations):
        registration_id = registrations[0]
        Bed = Pool().get('gnuhealth.hospital.bed')

        cls.write(registrations, {'state': 'finished'})

        Bed.write([registration_id.bed], {'state': 'free'})

    @classmethod
    @ModelView.button
    def cancel(cls, registrations):
        registration_id = registrations[0]
        Bed = Pool().get('gnuhealth.hospital.bed')

        cls.write(registrations, {'state': 'cancelled'})
        Bed.write([registration_id.bed], {'state': 'free'})

    @classmethod
    @ModelView.button
    def admission(cls, registrations):
        registration_id = registrations[0]
        Bed = Pool().get('gnuhealth.hospital.bed')

        Company = Pool().get('company.company')

        timezone = None
        company_id = Transaction().context.get('company')
        if company_id:
            company = Company(company_id)
            if company.timezone:
                timezone = pytz.timezone(company.timezone)
                
                dt = datetime.today()
                dt_local = datetime.astimezone(dt.replace(tzinfo=pytz.utc),
                 timezone)        

                if (registration_id.hospitalization_date.date() !=
                    dt_local.date()):
                    cls.raise_user_error("The Admission date must be today")
                else:
                    cls.write(registrations, {'state': 'hospitalized'})
                    Bed.write([registration_id.bed], {'state': 'occupied'})

            else:
               cls.raise_user_error('need_timezone')

    @classmethod
    def create(cls, vlist):
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('name'):
                config = Config(1)
                values['name'] = Sequence.get_id(
                    config.inpatient_registration_sequence.id)
        return super(InpatientRegistration, cls).create(vlist)

    @staticmethod
    def default_state():
        return 'free'

    @classmethod
    def validate(cls, registrations):
        super(InpatientRegistration, cls).validate(registrations)
        for registration in registrations:
            registration.check_discharge_context()


    def check_discharge_context(self):
        if ((not self.discharge_reason or not self.discharge_dx
            or not self.admission_reason)
            and self.state == 'done'):
                self.raise_user_error('discharge_reason_needed')

    
    # Format Registration ID : Patient : Bed
    def get_rec_name(self, name):
        if self.patient:
            return self.name + ':' + self.bed.rec_name + ':' \
                + self.patient.rec_name 
                
        else:
            return self.name

    # Allow searching by the hospitalization code, patient name
    # or bed number 

    @classmethod
    def search_rec_name(cls, name, clause):
        field = None
        # Search by Registration Code ID or Patient
        for field in ('name', 'patient', 'bed'):
            registrations = cls.search([(field,) + tuple(clause[1:])], limit=1)
            if registrations:
                break
        if registrations:
            return [(field,) + tuple(clause[1:])]
        return [(cls._rec_name,) + tuple(clause[1:])]



        
class BedTransfer(ModelSQL, ModelView):
    'Bed transfers'
    __name__ = 'gnuhealth.bed.transfer'

    name = fields.Many2One('gnuhealth.inpatient.registration',
        'Registration Code')
    transfer_date = fields.DateTime('Date')
    bed_from = fields.Many2One('gnuhealth.hospital.bed', 'From',
        )
    bed_to = fields.Many2One('gnuhealth.hospital.bed', 'To',
        )
    reason = fields.Char('Reason')


class Appointment(ModelSQL, ModelView):
    __name__ = 'gnuhealth.appointment'

    inpatient_registration_code = fields.Many2One(
        'gnuhealth.inpatient.registration', 'Inpatient Registration',
        help="Enter the patient hospitalization code")

class PatientEvaluation(ModelSQL, ModelView):
    __name__ = 'gnuhealth.patient.evaluation'

    inpatient_registration_code = fields.Many2One(
        'gnuhealth.inpatient.registration', 'IPC',
        help="Enter the patient hospitalization code")

class ECG(ModelSQL, ModelView):
    __name__ = 'gnuhealth.patient.ecg'

    inpatient_registration_code = fields.Many2One(
        'gnuhealth.inpatient.registration', 'Inpatient Registration',
        help="Enter the patient hospitalization code")


class PatientData(ModelSQL, ModelView):
    'Inherit patient model and add the patient status to the patient.'
    __name__ = 'gnuhealth.patient'

    patient_status = fields.Function(fields.Boolean('Hospitalized',
        help="Show the hospitalization status of the patient"),
        getter='get_patient_status', searcher='search_patient_status')

    @classmethod
    def get_patient_status(cls, patients, name):
        cursor = Transaction().connection.cursor()
        pool = Pool()
        Registration = pool.get('gnuhealth.inpatient.registration')
        registration = Registration.__table__()

        # Will store statuses {patient: True/False, ...}
        ids = list(map(int, patients))
        result = dict.fromkeys(ids, False)

        for sub_ids in grouped_slice(ids):
            # SQL expression for relevant patient ids
            clause_ids = reduce_ids(registration.id, sub_ids)

            # Hospitalized patient ids
            query = registration.select(registration.patient, Literal(True),
                where=(registration.state == 'hospitalized') & clause_ids,
                group_by=registration.patient)

            # Update dictionary of patient ids with True statuses
            cursor.execute(*query)
            result.update(cursor.fetchall())

        return result

    @classmethod
    def search_patient_status(cls, name, clause):
        p = Pool().get('gnuhealth.inpatient.registration')
        table = p.__table__()
        pat = cls.__table__()
        _, operator, value = clause

        # Validate operator and value
        if operator not in ['=', '!=']:
            raise ValueError('Wrong operator: %s' % operator)
        if value is not True and value is not False:
            raise ValueError('Wrong value: %s' % value)

        # Find hospitalized patient ids
        j = pat.join(table, condition = pat.id == table.patient)
        s = j.select(pat.id, where = table.state == 'hospitalized')

        # Choose domain operator
        if (operator == '=' and value) or (operator == '!=' and not value):
            d = 'in'
        else:
            d = 'not in'

        return [('id', d, s)]

class InpatientMedication (ModelSQL, ModelView):
    'Inpatient Medication'
    __name__ = 'gnuhealth.inpatient.medication'

    name = fields.Many2One('gnuhealth.inpatient.registration',
        'Registration Code')
    medicament = fields.Many2One('gnuhealth.medicament', 'Medicament',
        required=True, help='Prescribed Medicament')
    indication = fields.Many2One('gnuhealth.pathology', 'Indication',
        help='Choose a disease for this medicament from the disease list. It'
        ' can be an existing disease of the patient or a prophylactic.')
    start_treatment = fields.DateTime('Start',
        help='Date of start of Treatment', required=True)
    end_treatment = fields.DateTime('End', help='Date of start of Treatment')
    dose = fields.Float('Dose',
        help='Amount of medication (eg, 250 mg) per dose', required=True)
    dose_unit = fields.Many2One('gnuhealth.dose.unit', 'dose unit',
        required=True, help='Unit of measure for the medication to be taken')
    route = fields.Many2One('gnuhealth.drug.route', 'Administration Route',
        required=True, help='Drug administration route code.')
    form = fields.Many2One('gnuhealth.drug.form', 'Form', required=True,
        help='Drug form, such as tablet or gel')
    qty = fields.Integer('x', required=True,
        help='Quantity of units (eg, 2 capsules) of the medicament')
    common_dosage = fields.Many2One('gnuhealth.medication.dosage', 'Frequency',
        help='Common / standard dosage frequency for this medicament')
    admin_times = fields.One2Many('gnuhealth.inpatient.medication.admin_time',
        'name', "Admin times")
    log_history = fields.One2Many('gnuhealth.inpatient.medication.log', 'name',
        "Log History")
    frequency = fields.Integer('Frequency',
        help='Time in between doses the patient must wait (ie, for 1 pill'
        ' each 8 hours, put here 8 and select \"hours\" in the unit field')
    frequency_unit = fields.Selection([
        (None, ''),
        ('seconds', 'seconds'),
        ('minutes', 'minutes'),
        ('hours', 'hours'),
        ('days', 'days'),
        ('weeks', 'weeks'),
        ('wr', 'when required'),
        ], 'unit', select=True, sort=False)
    frequency_prn = fields.Boolean('PRN',
        help='Use it as needed, pro re nata')
    is_active = fields.Boolean('Active',
        help='Check if the patient is currently taking the medication')
    discontinued = fields.Boolean('Discontinued')
    course_completed = fields.Boolean('Course Completed')
    discontinued_reason = fields.Char('Reason for discontinuation',
        states={
            'invisible': Not(Bool(Eval('discontinued'))),
            'required': Bool(Eval('discontinued')),
            },
        depends=['discontinued'],
        help='Short description for discontinuing the treatment')
    adverse_reaction = fields.Text('Adverse Reactions',
        help='Side effects or adverse reactions that the patient experienced')

    @fields.depends('discontinued', 'course_completed')
    def on_change_with_is_active(self):
        is_active = True
        if (self.discontinued or self.course_completed):
            is_active = False
        return is_active

    @fields.depends('is_active', 'course_completed')
    def on_change_with_discontinued(self):
        return not (self.is_active or self.course_completed)

    @fields.depends('is_active', 'discontinued')
    def on_change_with_course_completed(self):
        return not (self.is_active or self.discontinued)

    @staticmethod
    def default_is_active():
        return True


class InpatientMedicationAdminTimes (ModelSQL, ModelView):
    'Inpatient Medication Admin Times'
    __name__="gnuhealth.inpatient.medication.admin_time"

    name = fields.Many2One('gnuhealth.inpatient.medication', 'Medication')
    admin_time = fields.Time("Time")
    dose = fields.Float('Dose',
        help='Amount of medication (eg, 250 mg) per dose')
    dose_unit = fields.Many2One('gnuhealth.dose.unit', 'dose unit',
        help='Unit of measure for the medication to be taken')
    remarks = fields.Text('Remarks',
        help='specific remarks for this dose')


class InpatientMedicationLog (ModelSQL, ModelView):
    'Inpatient Medication Log History'
    __name__="gnuhealth.inpatient.medication.log"

    name = fields.Many2One('gnuhealth.inpatient.medication', 'Medication')
    admin_time = fields.DateTime("Date", readonly=True)
    health_professional = fields.Many2One('gnuhealth.healthprofessional',
        'Health Professional', readonly=True)
    dose = fields.Float('Dose',
        help='Amount of medication (eg, 250 mg) per dose')
    dose_unit = fields.Many2One('gnuhealth.dose.unit', 'dose unit',
        help='Unit of measure for the medication to be taken')
    remarks = fields.Text('Remarks',
        help='specific remarks for this dose')

    @classmethod
    def __setup__(cls):
        super(InpatientMedicationLog, cls).__setup__()
        cls._error_messages.update({
            'health_professional_warning':
                    'No health professional associated to this user',
        })

    @classmethod
    def validate(cls, records):
        super(InpatientMedicationLog, cls).validate(records)
        for record in records:
            record.check_health_professional()

    def check_health_professional(self):
        if not self.health_professional:
            self.raise_user_error('health_professional_warning')

    @staticmethod
    def default_health_professional():
        pool = Pool()
        return pool.get('gnuhealth.healthprofessional').get_health_professional()

    @staticmethod
    def default_admin_time():
        return datetime.now()


class InpatientDiet (ModelSQL, ModelView):
    'Inpatient Diet'
    __name__="gnuhealth.inpatient.diet"

    name = fields.Many2One('gnuhealth.inpatient.registration',
        'Registration Code')
    diet = fields.Many2One('gnuhealth.diet.therapeutic', 'Diet', required=True)
    remarks = fields.Text('Remarks / Directions',
        help='specific remarks for this diet / patient')

class InpatientMeal (ModelSQL, ModelView):
    'Inpatient Meal'
    __name__="gnuhealth.inpatient.meal"

    name = fields.Many2One(
        'product.product', 'Food', required=True,
        help='Food')

    diet_therapeutic = fields.Many2One('gnuhealth.diet.therapeutic',
        'Diet')

    diet_belief = fields.Many2One('gnuhealth.diet.belief',
        'Belief')

    diet_vegetarian = fields.Many2One('gnuhealth.vegetarian_types',
        'Vegetarian')

    institution = fields.Many2One('gnuhealth.institution', 'Institution')

    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        institution = HealthInst.get_institution()
        return institution

    def get_rec_name(self, name):
        if self.name:
            return self.name.name

class InpatientMealOrderItem (ModelSQL, ModelView):
    'Inpatient Meal Item'
    __name__="gnuhealth.inpatient.meal.order.item"

    name = fields.Many2One('gnuhealth.inpatient.meal.order',
        'Meal Order')

    meal = fields.Many2One('gnuhealth.inpatient.meal', 'Meal')

    remarks = fields.Char('Remarks')


class InpatientMealOrder (ModelSQL, ModelView):
    'Inpatient Meal Order'
    __name__="gnuhealth.inpatient.meal.order"

    name = fields.Many2One('gnuhealth.inpatient.registration',
        'Registration Code', domain=[('state', '=', 'hospitalized')],
        required=True)

    mealtime = fields.Selection((
        (None, ''),
        ('breakfast', 'Breakfast'),
        ('lunch', 'Lunch'),
        ('dinner', 'Dinner'),
        ('snack', 'Snack'),
        ('special', 'Special order'),
        ), 'Meal time', required=True, sort=False)

    meal_item = fields.One2Many('gnuhealth.inpatient.meal.order.item', 'name',
        'Items')

    meal_order = fields.Char('Order', readonly=True)

    health_professional = fields.Many2One('gnuhealth.healthprofessional',
        'Health Professional')

    remarks = fields.Text('Remarks')

    meal_warning = fields.Boolean('Warning',
        help="The patient has special needs on meals")

    meal_warning_ack = fields.Boolean('Ack',
        help="Check if you have verified the warnings on the"
        " patient meal items")

    order_date = fields.DateTime('Date',
        help='Order date', required=True)

    state = fields.Selection((
        (None, ''),
        ('draft', 'Draft'),
        ('cancelled', 'Cancelled'),
        ('ordered', 'Ordered'),
        ('processing', 'Processing'),
        ('done','Done'),
        ), 'Status', readonly=True)

    @staticmethod
    def default_order_date():
        return datetime.now()

    @staticmethod
    def default_state():
        return 'draft'

    @staticmethod
    def default_health_professional():
        pool = Pool()
        return pool.get('gnuhealth.healthprofessional').get_health_professional()

    @classmethod
    def create(cls, vlist):
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('meal_order'):
                config = Config(1)
                values['meal_order'] = Sequence.get_id(
                    config.inpatient_meal_order_sequence.id)
        return super(InpatientMealOrder, cls).create(vlist)

    @classmethod
    def validate(cls, meal_orders):
        super(InpatientMealOrder, cls).validate(meal_orders)
        for meal_order in meal_orders:
            meal_order.check_meal_order_warning()
            meal_order.check_health_professional()
            
    def check_meal_order_warning(self):
        if not self.meal_warning_ack and self.meal_warning:
            self.raise_user_error('meal_order_warning')

    def check_health_professional(self):
        if not self.health_professional:
            self.raise_user_error('health_professional_warning')


    @fields.depends('name')
    def on_change_name(self):
        if self.name:
            # Trigger the warning if the patient 
            # has special needs on meals (religion / philosophy )
            if (self.name.patient.vegetarian_type or 
                self.name.patient.diet_belief):
                self.meal_warning = True
                self.meal_warning_ack = False


    @classmethod
    def __setup__(cls):
        super(InpatientMealOrder, cls).__setup__()
        cls._buttons.update({
            'cancel': {'invisible': Not(Equal(Eval('state'), 'ordered'))}
            })

        cls._buttons.update({
            'generate': {'invisible': Or(Equal(Eval('state'), 'ordered'),
                    Equal(Eval('state'), 'done'))},
            })

        cls._buttons.update({
            'done': {'invisible': Not(Equal(Eval('state'), 'ordered'))}
            })

        cls._error_messages.update({
            'meal_order_warning':
            '===== MEAL WARNING ! =====\n\n\n'
            'This patient has special meal needs \n\n'
            'Check and acknowledge that\n'
            'the meal items in this order are correct \n\n',
            'health_professional_warning':
            'No health professional associated to this user',
            })

        t = cls.__table__()
        cls._sql_constraints = [
            ('meal_order_uniq', Unique(t,t.meal_order),
                'The Meal Order code already exists'),
            ]


    @classmethod
    @ModelView.button
    def generate(cls, mealorders):
        cls.write(mealorders, {
            'state': 'ordered'})

    @classmethod
    @ModelView.button
    def cancel(cls, mealorders):
        cls.write(mealorders, {
            'state': 'cancelled'})

    @classmethod
    @ModelView.button
    def done(cls, mealorders):
        cls.write(mealorders, {
            'state': 'done'})



#  gnuhealth_csv_lab_interface.py
#  
#  Copyright 2017 Luis Falcon <falcon@gnu.org>
#  Copyright 2011-2017 GNU Solidario <health@gnusolidario.org>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  

# Check the README file for documentation on how to use this program

from proteus import config, Model
import csv
import sys

dbname = 'sandbox32'
user = 'admin'
password = 'gnusolidario'
hostname = 'localhost'
port = '8069'

health_server = \
    'http://'+user+':'+password+'@'+hostname+':'+port+'/'+dbname

def check_lab_test():
    LabTest = Model.get('gnuhealth.lab')
    LabTestLine = Model.get('gnuhealth.lab.test.critearea')

    # Verify that each test exists at the DB before trying 
    # to upload the results
    csv_file = csv.reader(open(sys.argv[1], 'rb'))
    for line in csv_file:
        test_id = line[0]
        analyte = line[1]
        if not (LabTest.find(['name','=',test_id])):
            exit("ERROR: Test %s not found" %test_id)
            
        if  not (LabTestLine.find([('name','=',analyte), \
            ('gnuhealth_lab_id','=',test_id)])):
            exit("ERROR: Analyte %s not found on %s" %(analyte, test_id))

    print "Basic check on test ID and analytes succeeded"
            
def input_results():
    LabTestLine = Model.get('gnuhealth.lab.test.critearea')
    csv_file = csv.reader(open(sys.argv[1], 'rb'))
    for line in csv_file:
        test_id = line[0]
        analyte = line[1]
        result = line[2]
        # Update the model with the result values
        for result_line in LabTestLine.find([('name','=',analyte), \
            ('gnuhealth_lab_id','=',test_id)]):
            result_line.result = float(result)
            result_line.save()

if (len(sys.argv) < 2):
    exit ("You need to specify a CSV file with the lab results")
    
print "Connecting to GNU Health Server ..."
conf = config.set_xmlrpc(health_server)
print "Connected !"

print "Checking integrity of the batch file ..."
check_lab_test()
print "Updating lab results from batch file ..."
input_results()
print "Done !"



from StringIO import StringIO

class ExportXMLMixin:
    def export_to_xml_string(self):
        """Export to string"""

        output = StringIO()
        self.export(outfile=output, namespacedef_='xmlns="http://hl7.org/fhir"', pretty_print=False, level=4)
        content = output.getvalue()
        output.close()
        return content






from .propfind import PROPFIND
from xml.dom import minidom
domimpl = minidom.getDOMImplementation()

from .utils import get_parenturi

class REPORT(PROPFIND):

    def __init__(self, uri, dataclass, depth, body):
        PROPFIND.__init__(self, uri, dataclass, depth, body)

        doc = minidom.parseString(body)

        self.filter = doc.documentElement

    def create_propname(self):
        """ create a multistatus response for the prop names """

        dc=self._dataclass
        # create the document generator
        doc = domimpl.createDocument(None, "multistatus", None)
        ms = doc.documentElement
        ms.setAttribute("xmlns:D", "DAV:")
        ms.tagName = 'D:multistatus'

        if self._depth=="0":
            if self._uri in self._dataclass.get_childs(get_parenturi(self._uri),
                    self.filter):
                pnames=dc.get_propnames(self._uri)
                re=self.mk_propname_response(self._uri,pnames, doc)
                ms.appendChild(re)

        elif self._depth=="1":
            if self._uri in self._dataclass.get_childs(get_parenturi(self._uri),
                    self.filter):
                pnames=dc.get_propnames(self._uri)
                re=self.mk_propname_response(self._uri,pnames, doc)
                ms.appendChild(re)

            for newuri in dc.get_childs(self._uri, self.filter):
                pnames=dc.get_propnames(newuri)
                re=self.mk_propname_response(newuri,pnames, doc)
                ms.appendChild(re)
        elif self._depth=='infinity':
            uri_list = [self._uri]
            while uri_list:
                uri = uri_list.pop()
                if uri in self._dataclass.get_childs(get_parenturi(uri),
                        self.filter):
                    pnames=dc.get_propnames(uri)
                    re=self.mk_propname_response(uri,pnames, doc)
                    ms.appendChild(re)
                uri_childs = self._dataclass.get_childs(uri)
                if uri_childs:
                    uri_list.extend(uri_childs)

        return doc.toxml(encoding="utf-8")

    def create_prop(self):
        """ handle a <prop> request

        This will

        1. set up the <multistatus>-Framework

        2. read the property values for each URI 
           (which is dependant on the Depth header)
           This is done by the get_propvalues() method.

        3. For each URI call the append_result() method
           to append the actual <result>-Tag to the result
           document.

        We differ between "good" properties, which have been
        assigned a value by the interface class and "bad" 
        properties, which resulted in an error, either 404
        (Not Found) or 403 (Forbidden).

        """


        # create the document generator
        doc = domimpl.createDocument(None, "multistatus", None)
        ms = doc.documentElement
        ms.setAttribute("xmlns:D", "DAV:")
        ms.tagName = 'D:multistatus'

        if self._depth=="0":
            if self._uri in self._dataclass.get_childs(get_parenturi(self._uri),
                    self.filter):
                gp,bp=self.get_propvalues(self._uri)
                res=self.mk_prop_response(self._uri,gp,bp,doc)
                ms.appendChild(res)

        elif self._depth=="1":
            if self._uri in self._dataclass.get_childs(get_parenturi(self._uri),
                    self.filter):
                gp,bp=self.get_propvalues(self._uri)
                res=self.mk_prop_response(self._uri,gp,bp,doc)
                ms.appendChild(res)

            for newuri in self._dataclass.get_childs(self._uri, self.filter):
                gp,bp=self.get_propvalues(newuri)
                res=self.mk_prop_response(newuri,gp,bp,doc)
                ms.appendChild(res)
        elif self._depth=='infinity':
            uri_list = [self._uri]
            while uri_list:
                uri = uri_list.pop()
                if uri in self._dataclass.get_childs(get_parenturi(uri),
                        self.filter):
                    gp,bp=self.get_propvalues(uri)
                    res=self.mk_prop_response(uri,gp,bp,doc)
                    ms.appendChild(res)
                uri_childs = self._dataclass.get_childs(uri)
                if uri_childs:
                    uri_list.extend(uri_childs)

        return doc.toxml(encoding="utf-8")




# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from trytond.pool import Pool
from .health_icu import *


def register():
    Pool.register(
        InpatientRegistration,
        InpatientIcu,
        Glasgow,
        ApacheII,
        MechanicalVentilation,
        ChestDrainageAssessment,
        PatientRounding,
        module='health_icu', type_='model')



from StringIO import StringIO
from operator import attrgetter
from .datastore import find_record
from server.common import safe_attrgetter
from. health_mixin import ExportXMLMixin
import server.fhir as supermod
import sys

try:
    from flask import current_app, url_for
    RUN_FLASK=True
except:
    from .datastore import dumb_url_generate
    RUN_FLASK=False

class Organization_Map:

    root_search = []
    resource_search_params={ '_id': 'token',
                            '_language': None,
                            'active': None,
                            'identifier': 'token',
                            'name': 'string',
                            'partof': None,
                            'phonetic': None,
                            'type': None}
    model_mapping = {
            'gnuhealth.institution':
                {
                    'name': 'name.name',
                    'identifier': 'code',
                    'type': 'institution_type',
                    'address': 'name.addresses',
                    'contacts': 'name.contact_mechanisms'
                    }
                }
    search_mapping={
            '_id': ['id'],
            'identifier': ['code'],
            'name': ['name.name']}
            #'type': ['institution_type']
            #}

class health_Organization(supermod.Organization, Organization_Map, ExportXMLMixin):
    """This subclass provides the glue between the schema generated bindings
    and GNU Health models. The Organization resource is robust, including
    healthcare providers, insurance companies, departments, etc. Currently,
    only GNU Health instutitions (gnuhealth.institution model) are supported.
    In the future, it is unclear whether there will be a need for other types 
    of organizations, but possibly units in the institution.
    """

    def __init__(self, *args, **kwargs):
        rec = kwargs.pop('gnu_record', None)
        super(health_Organization, self).__init__(*args, **kwargs)
        if rec:
            self.set_gnu_organization(rec)

    def set_gnu_organization(self, organization):
        """Set the organization

        Keyword arguments:
        organization - organization (Health class)
        """

        self.organization = organization
        self.model_type = self.organization.__name__

        if self.model_type not in self.model_mapping:
            raise ValueError('Not a valid model')

        self.map_ = self.model_mapping[self.model_type]

        self.__import_from_gnu_organization()
        self.__set_feed_info()

    def __import_from_gnu_organization(self):
        if self.organization:
            self.set_name(safe_attrgetter(self.organization,
                self.map_['name']))
            self.set_identifier(safe_attrgetter(self.organization,
                self.map_['identifier']))

            # Need selection text, not value
            t = self.map_['type']
            v = safe_attrgetter(self.organization, t)
            d = dict(self.organization.fields_get(t)[t]['selection'])
            self.set_type(d.get(v))

            self.set_address(safe_attrgetter(self.organization,
                self.map_['address'], default=[]))
            self.set_telecom(safe_attrgetter(self.organization,
                self.map_['contacts']))
            self.set_active(True)

    def __set_feed_info(self):
        """Set the feed-relevant data"""
        if self.organization:
            if RUN_FLASK:
                uri = url_for('org_record', log_id=self.organization.id,
                        _external=True)
            else:
                uri = dumb_url_generate(['Organization', self.organization.id])
            self.feed={'id': uri,
                'published': self.organization.create_date,
                'updated': self.organization.write_date or \
                        self.organization.create_date,
                'title': self.organization.name.rec_name
                        }

    def set_type(self, type_):
        """Extend superclass

        Set the institution's type

        Keyword arguments:
        type_ - type description
        """

        # TODO Use FHIR codes
        g=supermod.CodeableConcept()
        coding = supermod.Coding(
                system=supermod.uri(value='http://hl7.org/fhir/organization-type'),
                code=supermod.code(value='prov'),
                display=supermod.string(value='Healthcare Provider'))
        if type_:
            g.text = supermod.string(value=str(type_))
        g.coding = [coding]
        super(health_Organization, self).set_type(g)

    def set_identifier(self, identifier):
        """Extend superclass

        Set the institution's identifier

        Keyword arguments:
        identifier - institution code
        """

        if identifier is not None:
            ident = supermod.Identifier(
                        use=supermod.IdentifierUse(value='official'),
                        label=supermod.string(value='Institution code'),
                        value=supermod.string(value=str(identifier)))
            super(health_Organization, self).set_identifier([ident])

    def set_address(self, addresses):
        """Extend superclass

        Set the institution's address

        Keyword arguments:
        addresses - address models (party.address)
        """

        ads=[]
        for address in addresses:
            ad=supermod.Address()
            ad.set_use(supermod.string(value='work'))
            line=[]
            street, zip_, city, state, country = safe_attrgetter(
                                    address,
                                    'street',
                                    'zip',
                                    'city',
                                    'subdivision.name',
                                    'country.name')

            # Deal with number in field maybe never
            #if number:
                #line.append(str(number))
            if street:
                line.append(street)
            if line:
                ad.add_line(supermod.string(value=' '.join(line)))

            if city:
                ad.set_city(supermod.string(value=city))

            if state:
                ad.set_state(supermod.string(value=state))

            if zip_:
                ad.set_zip(supermod.string(value=zip_))

            if country:
                ad.set_country(supermod.string(value=country))

            ads.append(ad)

        super(health_Organization, self).set_address(ads)

    def set_telecom(self, contacts):
        """Extends superclass for convenience

        Set telecom information

        Keyword arguments:
        contacts -- contacts info (Party model)
        """

        telecom = []
        for contact in contacts:
            c=supermod.Contact()
            c.value = supermod.string(value=contact.value)
            if contact.type == 'phone':
                system='phone'
                use='work'
            else:
                use = system = contact.type
            c.system=supermod.ContactSystem(value=system)
            c.use=supermod.ContactUse(value=use)
            telecom.append(c)

        if telecom:
            super(health_Organization, self).set_telecom(telecom)

    def set_name(self, name):
        """Extend superclass

        Set the Organization's name

        Keyword arguments:
        name - organization name
        """

        if name:
            super(health_Organization, self).set_name(supermod.string(value=str(name)))

    def set_active(self, active=True):
        """Extend superclass

        Set the active status

        Keyword arguments:
        active - active status (boolean)
        """

        if active in [True, 'true']:
            d = supermod.boolean(value='true')
        else:
            d = supermod.boolean(value='false')
        super(health_Organization, self).set_active(d)

supermod.Organization.subclass = health_Organization
